import 'dart:math';
import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

extension ColorExt on Color {
  /// Darken color by given [factor]
  Color darken(double factor) => Color.lerp(this, Colors.black, factor)!;

  /// Brighten color by given [factor]
  Color brighten(double factor) => Color.lerp(this, Colors.white, factor)!;

  /// Compute RGB difference between this and other and [returns] delta
  double differenceRGB(Color other) =>
      sqrt(pow(other.red - red, 2) + pow(other.green - green, 2) + pow(other.blue - blue, 2));

  /// Short alias for [differenceRGB]
  double operator -(Color other) => differenceRGB(other);

  /// Get negative color in RGB scale
  Color get negative => Color.fromARGB(alpha, 255 - red, 255 - green, 255 - blue);

  /// [returns] brightness of this color based on luminance
  Brightness get brightness => computeLuminance() > .5 ? Brightness.light : Brightness.dark;

  /// [returns] HEX string representing this color e.g. FF00BB11
  String get hexString => value.toRadixString(16).padLeft(8, '0').toUpperCase();
}
