import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

void main() => group('Flutter Extension', () {
      test('testIterable', testIterable);
      test('testBlend', testBlend);
      test('testLightness', testLightness);
      test('testLoggable', testLoggable);
    });

void testIterable() {
  var list = ['Hello', 'World'];
  expect(list.countWhere((item) => item.contains('W')), 1);
  expect(list.count('World'), 1);
  expect(list.any((item) => item.contains('W')), true);
  expect(list.contains('World'), true);
  list.add('World');
  list.add('World');
  list.add('World');
  list.add('World');
  list.add('LoL');
  expect(list.count('World'), 5);
  list = list.toSet().toList();
  expect(list.count('World'), 1);
  list = list.distinct((item) => item.contains('o')).toList();
  expect(list.length, 1);
  list.addIfAbsent('World');
  list.addIfAbsent('World');
  list.addIfAbsent('World');
  list.addIfAbsent('World');
  expect(list.count('World'), 1);
  expect(MyEnum.values.fromName('due'), MyEnum.due);
  expect(MyEnum.values.fromName('sei', fallback: MyEnum.tre), MyEnum.tre);
  expect(MyEnum.values.require((item) => item.toString() == 'MyEnum.due', message: () => 'oh no'), MyEnum.due);
  final json = {'value': 'tre'};
  expect(json.getEnum('value', MyEnum.values), MyEnum.tre);
  final values = [102, 33, 4, 6, 1, 5, -30, 6, 7, 88, 9.0];
  expect(values.closestTo(17), 9);
  expect(values.closestTo(10), 9);
  expect(values.closestTo(7.4), 7);
  expect(values.closestTo(-10), 1);
  expect(values.closestTo(-20), -30);
  expect(values.closestTo(99920), 102);
}

enum MyEnum {
  uno,
  due,
  tre,
}

void testBlend() {
  const red = Color(0xffff0000);
  const yellow = Color(0xffffff00);
  const blue = Color(0xff0000ff);
  expect(Color.lerp(yellow, red, .5), const Color(0xffff7f00));
  expect(Color.lerp(blue, Colors.white, .6), const Color(0xff9999ff));
  expect(Color.lerp(blue, Colors.white, .2), const Color(0xff3333ff));
}

void testLightness() {
  final grey87 = Color.lerp(Colors.white, Colors.black, .87);
  expect(grey87, const Color(0xff212121));
  expect(grey87, equals(Color.lerp(Colors.white, Colors.black, .87)));
  expect(Colors.black.brighten(.13), grey87);
  expect(Colors.white.darken(.87), grey87);
  expect(Colors.white.darken(.5), Colors.black.brighten(.5));

  final blue = Colors.blue[500]!;
  expect(blue.brighten(1), Colors.white);
  expect(blue.brighten(.9), const Color(0xffe8f4fd));
  expect(blue.brighten(.8), const Color(0xffd2eafc));
  expect(blue.brighten(.7), const Color(0xffbcdffb));
  expect(blue.brighten(.6), const Color(0xffa6d5fa));
  expect(blue.brighten(.5), const Color(0xff90caf9));
  expect(blue.brighten(.4), const Color(0xff79c0f7));
  expect(blue.brighten(.3), const Color(0xff63b5f6));
  expect(blue.brighten(.2), const Color(0xff4dabf5));
  expect(blue.brighten(.1), const Color(0xff37a0f4));
  expect(blue.brighten(.0), blue);
  expect(blue.darken(.0), blue);
  expect(blue.darken(.1), const Color(0xFF1D87DA));
  expect(blue.darken(.2), const Color(0xFF1A78C2));
  expect(blue.darken(.3), const Color(0xFF1769AA));
  expect(blue.darken(.4), const Color(0xFF135A91));
  expect(blue.darken(.5), const Color(0xFF104B79));
  expect(blue.darken(.6), const Color(0xFF0D3C61));
  expect(blue.darken(.7), const Color(0xFF092D48));
  expect(blue.darken(.8), const Color(0xFF061E30));
  expect(blue.darken(.9), const Color(0xFF030F18));
  expect(blue.darken(1), Colors.black);
}

void testLoggable() => LoggableTest().start();

class LoggableTest with Loggable {
  void start() => log('Welcome to loggable test', () {
        log('Indented text');
        var result = const Result.success('Hello');
        log(result);
        log(result.resultOrThrow);
        result = Result.wrap(() => throw ArgumentError('Invalid argument'));
        log(result);
      });

  @override
  void logImpl(String message, [Object? error, StackTrace? stack]) =>
      error == null ? debugPrint('$tag:$indent$message') : log('$message\n$error\n$stack');
}
